from metamenth.enumerations.abstract_enum import AbstractEnum


class FCUPipeSystem(AbstractEnum):
    """
    Piping system for FCUs

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    TWO_PIPE = "TwoPipe"
    FOUR_PIPE = "FourPipe"


