from metamenth.enumerations.abstract_enum import AbstractEnum


class DuctConnectionEntityType(AbstractEnum):
    """
    Duct connection object types

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    SOURCE = "Source"
    DESTINATION = "Destination"

