from metamenth.enumerations.abstract_enum import AbstractEnum


class EngineSubType(AbstractEnum):
    """
    Modes of Engine

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    DIESEL = "Diesel"
    BIO_DIESEL = "Biodiesel"
    NATURAL_GAS = "NaturalGas"
    HYDROGEN = "Hydrogen"
    PETROL = "Petrol"
    COAL = "Coal"
    PROPANE = "Propane"
    ETHANOL = "Ethanol"
    GEO_THERMAL = "GeoThermal"
    GASOLINE = "GasOnline"
    JETFUEL = "JetFuel"
    WOOD = "Wood"
    BIOMASS = "Biomass"
    CHARCOAL = "Charcoal"
    PEAT = "Peat"
    OIL = "Oil"
    NONE = "None"
    OTHER = "Other"
