from metamenth.enumerations.abstract_enum import AbstractEnum


class WaveForm(AbstractEnum):
    """
    Different types of waveforms

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    SINUSOIDAL = "Sinusoidal"
    SQUARE = "Square"
    MODIFIED_SINE = "ModifiedSine"
    OTHER = "Other"

