from metamenth.enumerations.abstract_enum import AbstractEnum


class MeterMeasureMode(AbstractEnum):
    """
    Different types of floors in a building.

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    MANUAL = "Manual"
    AUTOMATIC = "Automatic"
