from metamenth.enumerations.abstract_enum import AbstractEnum


class ATSOperationMode(AbstractEnum):
    """
    ATS Operation modes

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    MANUAL = "Manual"
    NON_AUTOMATIC = "NonAutomatic"
    AUTOMATIC = "Automatic"
