from metamenth.enumerations.abstract_enum import AbstractEnum


class RadiantSlabType(AbstractEnum):
    """
    Types of radiant slabs

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    ELECTRIC = "Electric"
    AIR_HEATED = "AirHeated"
    HYDRONIC = "Hydronic"
    OTHER = "Other"

