from metamenth.enumerations.abstract_enum import AbstractEnum


class EnergySource(AbstractEnum):
    """
    Types of Energy Sources

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    GRID = "Grid"
    Renewable = "Renewable"
