from metamenth.enumerations.abstract_enum import AbstractEnum


class ATSTransitionType(AbstractEnum):
    """
    ATS transition types

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    OPEN = "Open"
    CLOSE = "Close"
