from enum import Enum


class CoverType(Enum):
    """
    The different types of building cover

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    ROOF = "Roof"
    WALL = "Wall"
    FLOOR = "Floor"
    WINDOW = "Window"
    DOOR = "Door"
