from metamenth.enumerations.abstract_enum import AbstractEnum


class FCUType(AbstractEnum):
    """
    Types of fan coil units

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    CEILING_MOUNTED = "CeilingMounted"
    STANDALONE = "Standalone"


