from enum import Enum


class ZoneType(Enum):
    """
    Different types of zones in a building.

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    HVAC = "HVAC"
    LIGHTING = "Lighting"
