from metamenth.enumerations.abstract_enum import AbstractEnum


class ATSSwitchingMechanism(AbstractEnum):
    """
    ATS Switching mechanisms

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    CONTRACTOR = "Contractor"
    MOLDED_CASE = "MoldedCase"
    POWER_FRAME = "PowerFrame"
