from metamenth.enumerations.abstract_enum import AbstractEnum


class ChillerType(AbstractEnum):
    """
    Types of chillers

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    WATER_COOLED = "WaterCooled"
    AIR_COOLED = "AirCooled"
    OTHER = "Other"

