from metamenth.enumerations.abstract_enum import AbstractEnum


class ApplianceCategory(AbstractEnum):
    """
    Different categories of appliances

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    HOME = "Home"
    OFFICE = "Office"
    SMART = "Smart"
    OTHER = "Other"
    NONE = None

