from metamenth.enumerations.abstract_enum import AbstractEnum


class HeatingType(AbstractEnum):
    """
    Types of heating

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    ELECTRIC = "Electric"
    HYDRONIC = "Hydronic"
    DIMPLEX = "Dimplex"
    OTHER = "Other"

