from metamenth.enumerations.abstract_enum import AbstractEnum


class ApplianceType(AbstractEnum):
    """
    Different types of appliances

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    LIGHT_BULB = "LightBulb"
    KETTLE = "Kettle"
    MICROWAVE = "Microwave"
    REFRIGERATOR = "Refrigerator"
    FREEZER = "Freezer"
    THERMOSTAT = "Thermostat"
    COFFEE_MAKER = "CoffeeMaker"
    DOOR_BELL = "DoorBell"
    LOCK = "Lock"
    OVEN = "Oven"
    AIR_CONDITIONER = "AirConditioner"
    PLUG = "Plug"
    CAMERA = "Camera"
    DISHWASHER = "Dishwasher"
    TOASTER = "Toaster"
    RICE_COOKER = "RiceCooker"
    WASHING_MACHINE = "WashingMachine"
    TELEVISION = "Television"
    DRYER = "Dryer"
    PRESSING_IRON = "PressingIron"
    ELECTRIC_FAN = "Electric"
    STOVE = "Stove"
    BLENDER = "Blender"
    PRINTER = "Printer"
    SCANNER = "Scanner"
    PAPER_SHREDDER = "PaperShredder"
    LAMINATOR = "Laminator"
    PHOTOCOPIER = "Photocopier"
    SMOKE_DETECTOR = "SmokeDetector"
    VACUUM_CLEANER = "VacuumCleaner"
    PROJECTOR = "Projector"
    INSTANT_POT = "InstantPot"
    LAPTOP = "Laptop"
    DESKTOP_COMPUTER = "DesktopComputer"
    SECURITY_CAMERA = "SecurityCamera"
    POWER_STRIP = "PowerStrip"
    SWITCH = "Switch"
    WHITEBOARD = "Whiteboard"
    NOTEBOOK = "Notebook"
    AIR_PURIFIER = "AirPurifier"
    HUMIDIFIER = "Humidifier"
    DEHUMIDIFIER = "Dehumidifier"
    SPEAKER = "Speaker"
    OTHER = "Other"

