from metamenth.enumerations.abstract_enum import AbstractEnum


class VentilationType(AbstractEnum):
    """
    Different kinds of ventilation system

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    AIR_HANDLING_UNIT = "AirHandlingUnit"

