from metamenth.enumerations.abstract_enum import AbstractEnum


class HeatExchangerFlowType(AbstractEnum):
    """
    Heat exchanger flow types

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    PARALLEL = "Parallel"
    COUNTER = "Counter"
    OTHER = "Other"

