from metamenth.enumerations.abstract_enum import AbstractEnum


class BatteryTech(AbstractEnum):
    """
    Battery technologies

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    LITHIUM = "Lithium"
    RUTHENIUM = "Ruthenium"
    NICKEL = "Nickel"
    OTHER = "Other"
