from metamenth.enumerations.abstract_enum import AbstractEnum


class AirVolumeType(AbstractEnum):
    """
    Types of air volume boxes

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    VARIABLE_AIR_VOLUME = "VariableAirVolume"
    CONSTANT_AIR_VOLUME = "ConstantAirVolume"

