from metamenth.enumerations.abstract_enum import AbstractEnum


class FloorType(AbstractEnum):
    """
    Different types of floors in a building.

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    REGULAR = "Regular"
    BASEMENT = "Basement"
    ROOFTOP = "Rooftop"
