from metamenth.enumerations.abstract_enum import AbstractEnum


class TerrainType(AbstractEnum):
    """
   Terrain types of buildings

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    SUBURBS = "Suburbs"
    COUNTRY = "Country"
    CITY = "City"
    OCEAN = "Ocean"
    URBAN = "Urban"

