from metamenth.enumerations.abstract_enum import AbstractEnum


class BuildingType(AbstractEnum):
    """
    Different types of buildings.

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    COMMERCIAL = "Commercial"
    NON_COMMERCIAL = "NonCommercial"
    RESIDENTIAL = "Residential"

