from metamenth.enumerations.abstract_enum import AbstractEnum


class RefrigerantType(AbstractEnum):
    R134A = "R-134a"
    R410A = "R-410A"
    R22 = "R-22"
    R404A = "R-404A"
    R1234YF = "R-1234yf"
    R1234ZE = "R-1234ze"

