from metamenth.enumerations.abstract_enum import AbstractEnum


class CapacitorTech(AbstractEnum):
    """
    Technology used in super capacitors

    Author: Peter Yefi
    Email: peteryefi@gmail.com
    """
    LEAD_BATTERY = "LeadBattery"
    LITHIUM_IRON = "LithiumIron"
    NICKEL = "Nickel"